#!/bin/sh
#usb disk set
killall uvc-gadget
sync
sync
sync
rmmod g_webcam.ko

# Do not hardcode kernel
MODULE_DIR=$(find /lib/modules -maxdepth 1 -type d -name "$(uname -r)*" -print | head -1)
cd $MODULE_DIR/kernel/drivers/usb/gadget/

insmod u_serial.ko
insmod usb_f_acm.ko
insmod usb_f_mass_storage.ko
insmod g_acm_ms.ko file=/dev/mmcblk0p1 removable=1 stall=1

# If patch not yet applied
if grep -q "ttyGS0" /etc/securetty; then
  echo "System already patched"
else 
  echo "ttyGS0" >> /etc/securetty
  echo "root:password" | chpasswd
fi

# Run a TTY on CDC ACM USB
getty -L 115200 ttyGS0 &

# Identify partitions for backups
INTERNAL_DEV=""
EXTERNAL_SD_PART=""

EXT3_DEV=$(mount | sed -rn "s/.*\/dev\/([^/]+)p[0-9]+.*ext3.*/\1/p")
FAT_DEV=$(mount | sed -rn "s/.*\/dev\/([^/]+)p[0-9]+.*vfat.*/\1/p")

# Internal flash contain an EXT3 partition - skip EXT3 from SD if any
IFS='
'
count=0
for line in $EXT3_DEV; do
    count=$((count+1))
    if grep -q "SD" /sys/block/$line/device/type; then
        echo "EXT3 detected on SD Card - ignored." > /dev/ttyGS0
    else 
        INTERNAL_DEV=$EXT3_DEV
        echo $EXT3_DEV > /dev/ttyGS0
    fi
done

# We want to store on an FAT partition on external SD
IFS='
'
count=0
for line in $FAT_DEV; do
    count=$((count+1))
    if grep -q "SD" /sys/block/$line/device/type; then
        EXTERNAL_SD_PART=$(mount | sed -rn 's/.*\/dev\/('"$line"'p[0-9]+).*vfat.*/\1/p' | head -1)
        echo $EXTERNAL_SD_PART > /dev/ttyGS0
    else
        INTERNAL_DEV_FAT_PART=$(mount | sed -rn 's/.*\/dev\/('"$line"'p[0-9]+).*vfat.*/\1/p' | head -1)
        echo $INTERNAL_DEV_FAT_PART > /dev/ttyGS0
    fi
done


# Backup application files if a Backup folder is present on the SD
BACKUP_DIR="/media/$EXTERNAL_SD_PART/Backup"
FULL_BACKUP_DIR="/media/$EXTERNAL_SD_PART/FullBackup"
FORCEROOT_DIR="/media/$EXTERNAL_SD_PART/ForceRoot"

if [ -d "$FORCEROOT_DIR" ]; then
  echo "root:password" | chpasswd
fi

if [ -d "$BACKUP_DIR" ]; then
  echo "Performing backup of application files..." > /dev/ttyGS0
  cp /root/* $BACKUP_DIR/
  cp /etc/rc.d/* $BACKUP_DIR/
  cp /media/$INTERNAL_DEV_FAT_PART/zImage $BACKUP_DIR/
  cp /media/$INTERNAL_DEV_FAT_PART/*.dtb $BACKUP_DIR/
  echo "Done." > /dev/ttyGS0
fi

if [ -d "$FULL_BACKUP_DIR" ]; then
  echo "Performing full backup..."
  dd if=/dev/$INTERNAL_FLASH of=$FULL_BACKUP_DIR/image.bin bs=1M
  echo "Done." > /dev/ttyGS0
fi

cd -

